/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.wood;


import java.util.Random;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.ITileEntityProvider;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.*;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.client.TFCGuiHandler;
import net.dries007.tfc.objects.blocks.BlockCharcoalPile;
import net.dries007.tfc.objects.te.TELogPile;
import net.dries007.tfc.objects.te.TESidedInventory;
import net.dries007.tfc.util.Helpers;

@ParametersAreNonnullByDefault
@MethodsReturnNonnullByDefault
public class BlockLogPile extends Block implements ITileEntityProvider
{
    public static final PropertyBool ONFIRE = PropertyBool.func_177716_a("onfire");
    private static final PropertyBool AXIS = PropertyBool.func_177716_a("axis");

    public BlockLogPile()
    {
        super(Material.field_151575_d);

        func_149711_c(2.0F);
        func_149672_a(SoundType.field_185848_a);
        func_149675_a(true);
        setHarvestLevel("axe", 0);
        this.func_180632_j(this.func_176223_P().func_177226_a(AXIS, false).func_177226_a(ONFIRE, false));
    }

    @Override
    public TileEntity func_149915_a(World worldIn, int meta)
    {
        return new TELogPile();
    }

    @Override
    @SuppressWarnings("deprecation")
    public IBlockState func_176203_a(int meta)
    {
        return this.func_176223_P().func_177226_a(AXIS, meta == 0).func_177226_a(ONFIRE, meta >= 2);
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return (state.func_177229_b(AXIS) ? 0 : 1) + (state.func_177229_b(ONFIRE) ? 2 : 0);
    }

    @Override
    public void func_180645_a(World worldIn, BlockPos pos, IBlockState state, Random random)
    {
        if (!worldIn.field_72995_K && state.func_177229_b(ONFIRE))
        {
            for (EnumFacing side : EnumFacing.values())
            {
                if (!isValidCoverBlock(worldIn, pos.func_177972_a(side)))
                {
                    worldIn.func_175656_a(pos, Blocks.field_150480_ab.func_176223_P());
                }
                IBlockState state2 = worldIn.func_180495_p(pos.func_177972_a(side));
                if (state2.func_177230_c() instanceof BlockLogPile)
                {
                    if (!state2.func_177229_b(ONFIRE))
                    {
                        worldIn.func_175656_a(pos.func_177972_a(side), state2.func_177226_a(ONFIRE, true));
                    }
                }
            }
        }
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void func_180655_c(IBlockState stateIn, World worldIn, BlockPos pos, Random rand)
    {
        if (stateIn.func_177229_b(ONFIRE))
        {
            worldIn.func_175688_a(EnumParticleTypes.SMOKE_NORMAL, pos.func_177958_n() + rand.nextFloat(), pos.func_177956_o() + 1, pos.func_177952_p() + rand.nextFloat(),
                0f, 0.1f + 0.1f * rand.nextFloat(), 0f);
            if (rand.nextDouble() < 0.4D)
            {
                worldIn.func_184134_a((double) pos.func_177958_n() + 0.5D, (double) pos.func_177956_o(), (double) pos.func_177952_p() + 0.5D, SoundEvents.field_187643_bs, SoundCategory.BLOCKS, 0.5F, 0.6F, false);
            }
        }
    }

    @Override
    public boolean func_180639_a(World world, BlockPos pos, IBlockState state, EntityPlayer player, EnumHand hand, EnumFacing side, float hitX, float hitY, float hitZ)
    {
        if (!world.field_72995_K)
        {
            TELogPile te = (TELogPile) world.func_175625_s(pos);
            if (te == null) { return true; }

            // Special Interactions
            // 1. Try and put a log inside (happens on right click event)
            // 2. Try and light the TE
            // 3. Open the GUI
            ItemStack stack = player.func_184586_b(hand);
            if (stack.func_77973_b() == Items.field_151033_d && !state.func_177229_b(ONFIRE) && side == EnumFacing.UP)
            {
                // Light the Pile
                if (world.func_180495_p(pos.func_177984_a()).func_177230_c().func_176200_f(world, pos))
                {
                    world.func_175656_a(pos, state.func_177226_a(ONFIRE, true));
                    te.light();
                    world.func_175656_a(pos.func_177984_a(), Blocks.field_150480_ab.func_176223_P());
                    world.func_184133_a(null, pos, SoundEvents.field_187649_bu, SoundCategory.PLAYERS, 1.0F, 1.0F);
                }
            }

            if (!player.func_70093_af() && !state.func_177229_b(ONFIRE))
            {
                player.openGui(TerraFirmaCraft.getInstance(), TFCGuiHandler.LOG_PILE, world, pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p());
            }
        }
        return true;
    }

    @Override
    @SuppressWarnings("deprecation")
    public IBlockState func_180642_a(World worldIn, BlockPos pos, EnumFacing facing, float hitX, float hitY, float hitZ, int meta, EntityLivingBase placer)
    {
        if (placer.func_174811_aO().func_176740_k() == EnumFacing.Axis.Z)
        {
            return this.func_176223_P().func_177226_a(AXIS, true);
        }
        return this.func_176223_P();
    }

    @Override
    public void func_180657_a(World worldIn, EntityPlayer player, BlockPos pos, IBlockState state, @Nullable TileEntity te, ItemStack stack)
    {
        if (!worldIn.field_72995_K && te != null && te instanceof TESidedInventory)
        {
            ((TESidedInventory) te).onBreakBlock(worldIn, pos);
        }
        super.func_180657_a(worldIn, player, pos, state, te, stack);
    }

    @Override
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, AXIS, ONFIRE);
    }

    @Override
    public int getFlammability(IBlockAccess world, BlockPos pos, EnumFacing face)
    {
        return 60;
    }

    @Override
    public int getFireSpreadSpeed(IBlockAccess world, BlockPos pos, EnumFacing face)
    {
        return 30;
    }

    @Override
    public void getDrops(NonNullList<ItemStack> drops, IBlockAccess world, BlockPos pos, IBlockState state, int fortune)
    {
        drops.clear();
    }

    @Override
    public ItemStack getPickBlock(IBlockState state, RayTraceResult target, World world, BlockPos pos, EntityPlayer player)
    {
        TELogPile te = Helpers.getTE(world, pos, TELogPile.class);
        if (te != null)
        {
            return te.getLog().func_77946_l();
        }
        return ItemStack.field_190927_a;
    }

    private boolean isValidCoverBlock(World world, BlockPos pos)
    {
        IBlockState state = world.func_180495_p(pos);
        if (state.func_177230_c() instanceof BlockLogPile || state.func_177230_c() instanceof BlockCharcoalPile) { return true; }
        if (state.func_185904_a().func_76217_h()) { return false; }
        return state.func_185915_l();
    }

}
