/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.wood;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.BlockBush;
import net.minecraft.block.IGrowable;
import net.minecraft.block.ITileEntityProvider;
import net.minecraft.block.SoundType;
import net.minecraft.block.properties.PropertyInteger;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.api.types.Tree;
import net.dries007.tfc.objects.te.TESaplingTFC;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.OreDictionaryHelper;
import net.dries007.tfc.world.classic.CalenderTFC;

@ParametersAreNonnullByDefault
public class BlockSaplingTFC extends BlockBush implements IGrowable, ITileEntityProvider
{
    public static final PropertyInteger STAGE = PropertyInteger.func_177719_a("stage", 0, 4);
    protected static final AxisAlignedBB SAPLING_AABB = new AxisAlignedBB(0.09999999403953552D, 0.0D, 0.09999999403953552D, 0.8999999761581421D, 0.800000011920929D, 0.8999999761581421D);
    private static final Map<Tree, BlockSaplingTFC> MAP = new HashMap<>();

    public static BlockSaplingTFC get(Tree wood)
    {
        return MAP.get(wood);
    }

    public final Tree wood;

    public BlockSaplingTFC(Tree wood)
    {
        super();
        if (MAP.put(wood, this) != null) throw new IllegalStateException("There can only be one.");
        this.wood = wood;
        func_180632_j(field_176227_L.func_177621_b().func_177226_a(STAGE, 0));
        func_149672_a(SoundType.field_185850_c);
        func_149711_c(0.0F);
        OreDictionaryHelper.register(this, "tree", "sapling");
        OreDictionaryHelper.register(this, "tree", "sapling", wood.name());
        Blocks.field_150480_ab.func_180686_a(this, 5, 20);
    }

    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public IBlockState func_176203_a(int meta)
    {
        return this.func_176223_P().func_177226_a(STAGE, meta);
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return state.func_177229_b(STAGE);
    }

    @Override
    public int func_180651_a(IBlockState state)
    {
        return 0; // explicit override on default, because saplings should be reset when they are broken.
    }

    @Override
    public void func_176213_c(World worldIn, BlockPos pos, IBlockState state)
    {
        TESaplingTFC te = Helpers.getTE(worldIn, pos, TESaplingTFC.class);
        if (te != null) te.onPlaced();
    }

    @Override
    @Nonnull
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, STAGE);
    }

    @Override
    public void func_180650_b(World world, BlockPos pos, IBlockState state, Random random)
    {
        super.func_180650_b(world, pos, state, random);

        if (!world.field_72995_K)
        {
            TESaplingTFC te = Helpers.getTE(world, pos, TESaplingTFC.class);
            if (te != null)
            {
                long hours = te.getHoursSincePlaced();
                if (hours > wood.minGrowthTime * CalenderTFC.HOURS_IN_DAY)
                {
                    func_176474_b(world, random, pos, state);
                }
                // This is a hack to make saplings grow faster if the block underneath the dirt is glass. Useful for testing
                /*if (world.getBlockState(pos.down(2)) == Blocks.GLASS.getDefaultState())
                {
                    te.timer -= 10;
                    te.markDirty();
                    TerraFirmaCraft.getLog().info("Hacking the timer! New time:" + te.getHoursSincePlaced());
                }*/
            }
        }
    }

    @Override
    public boolean func_176473_a(World world, BlockPos blockPos, IBlockState iBlockState, boolean b)
    {
        return true;
    }

    @Override
    public boolean func_180670_a(World world, Random random, BlockPos blockPos, IBlockState iBlockState)
    {
        TerraFirmaCraft.getLog().info("canUseBoneMeal called");
        return true;
    }

    @Override
    @Nonnull
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        return SAPLING_AABB;
    }

    @Override
    public void func_176474_b(World world, Random random, BlockPos blockPos, IBlockState blockState)
    {
        wood.makeTree(world, blockPos, random);
    }

    @Nullable
    @Override
    public TileEntity func_149915_a(World worldIn, int meta)
    {
        return new TESaplingTFC();
    }
}
